import ddf.minim.*;
import ddf.minim.analysis.*;

Minim minim;
AudioInput myAudio;

// beat detection with sound energy
BeatDetect myBeat;
int myBeatMax = 20; // sync with gui
int myBeatCurrent = 0;
int myBeatPrevious = 0;

int beatKickMax = 20;
int beatKickCurrent = 0;
int beatKickPrevious = 0;

int beatSnareMax = 20;
int beatSnareCurrent = 0;
int beatSnarePrevious = 0;

int beatHatMax = 20;
int beatHatCurrent = 0;
int beatHatPrevious = 0;

FFT myFFT;
int fftDrawHeight = 200;
PGraphics spectrogram;
int specIndex = 0;

void setupAudioAnalysis() {
  minim = new Minim(this);
  myAudio = minim.getLineIn(); // Minim.STEREO, bufferSize=1024,  sampleRate=44100, int bitDepth=16

  // beat detection setup
  myBeat = new BeatDetect();

  // fft setup
  myFFT = new FFT(myAudio.bufferSize(), myAudio.sampleRate());
  myFFT.logAverages(22, 12); // (22,12) -> 120 averages
  myFFT.window(FFT.HANN);
  spectrogram = createGraphics(120*5, 130);

  // debug
  println("[debug][audio]: number of fft bins: "+myFFT.avgSize());
}

void drawAudioAnalysis() {
  // beat detection processing
  if (myBeatCurrent > 0) {
    myBeatCurrent--; // linear decay
  }
  myBeat.detect(myAudio.mix);
  if (myBeat.isOnset()) {
    myBeatCurrent = myBeatMax;
  }
  
  if (beatKickCurrent > 0) {
    beatKickCurrent--;
  }
  if (myBeat.isKick()) {
    beatKickCurrent = beatKickMax;
  }
  
  
  if (beatSnareCurrent > 0) {
    beatSnareCurrent--;
  }
  if (myBeat.isSnare()) {
    beatSnareCurrent = beatSnareMax;
  }
  
  
  
  if (beatHatCurrent > 0) {
    beatHatCurrent--;
  }
  if (myBeat.isHat()) {
    beatHatCurrent = beatHatMax;
  }
  
  // draw beat detection
  //fill(0);
  //noStroke();
  //rect(160, 40, 40, 40);
  fill(255);
  circle(160+20, 40+20, getBeatDetectNormalized()*36);
  circle(20+20, 500+20, map(beatKickCurrent, 0, beatKickMax, 10, 36));
  circle(20+20+40, 500+20, map(beatSnareCurrent, 0, beatSnareMax, 10, 36));
  circle(20+20+80, 500+20, map(beatHatCurrent, 0, beatHatMax, 10, 36));
  // send osc
  oscSendBeatValue(getBeatDetectNormalized());

  // fft processing
  myFFT.forward(myAudio.mix);
  drawFFT(220, 20);
  image(spectrogram, 220, 240);
}

/*******************************
 beat helper functions
 ********************************/
float getBeatDetectNormalized() {
  return (myBeatCurrent / float(myBeatMax));
}

void setBeatDecay(float val) {
  gui_beatDecay.setValue(map(val, 0, 1, gui_beatDecay.getStartLimit(), gui_beatDecay.getEndLimit()));
}

/********************************
 fft helper functions
 ********************************/
void setAmpBeginFraction(float val) {
  gui_fftAmpBeginFrac.setValue(map(val, 0, 1, gui_fftAmpBeginFrac.getStartLimit(), gui_fftAmpBeginFrac.getEndLimit()));
}

void setAmpStart(float val) {
  gui_ampStart.setValue(map(val, 0, 1, gui_ampStart.getStartLimit(), gui_ampStart.getEndLimit()));
}

void setAmpStep(float val) {
  gui_ampStep.setValue(map(val, 0, 1, gui_ampStep.getStartLimit(), gui_ampStep.getEndLimit()));
}


/*
FFT EXTRA
*/

int fftband1_start = 20;
int fftband1_width = 3;

float fftband1_force = 0.0;

int fftband2_start = 47;
int fftband2_width = 3;

float fftband2_force = 0.0;

// this also computes the calculated bands to save time in draw();
void drawFFT(int x, int y) {
  pushMatrix();
  translate(x, y);

  stroke(0);
  fill(255);

  int bins = 120; // should match myFFT.avgSize();
  int binwidth = 5;
  float myAudioIndexAmp = gui_baseAmp.getValueF();
  FloatList fftValues = new FloatList(120);
  
  float fftband1_tmp = 0;
  float fftband2_tmp = 0;
  for (int i = 0; i < bins; i++) {
    if (i < bins*gui_fftAmpBeginFrac.getValueF()) { // reset below thresh
      myAudioIndexAmp = gui_baseAmp.getValueF();
    }
    float tempIndexAvg = (myFFT.getAvg(i) * gui_baseAmp.getValueF()) * myAudioIndexAmp;
    float tempIndexCon = constrain(tempIndexAvg, 0, fftDrawHeight);
    rect((i*binwidth), fftDrawHeight, binwidth, -tempIndexCon);
    myAudioIndexAmp += gui_ampStep.getValueF();
    spectrogram.fill(tempIndexCon);
    fftValues.append(tempIndexCon);
    
    // process fttbanb1
    if(i >= fftband1_start && i < fftband1_start+fftband1_width) {
      fftband1_tmp += tempIndexCon;
    }
    if(i >= fftband2_start && i < fftband2_start+fftband2_width) {
      fftband2_tmp += tempIndexCon;
    }
  }
  fftband1_force = fftband1_tmp/fftband1_width;
  fftband2_force = fftband2_tmp/fftband2_width;
  
  if (frameCount % 5 == 0) {
    spectrogram.beginDraw();
    spectrogram.image(spectrogram, 0, 1);
    for (int i = 0; i < bins; i++) {
      spectrogram.stroke(fftValues.get(i));
      spectrogram.line(i*binwidth, specIndex, i*binwidth+binwidth, specIndex);
    }
    spectrogram.stroke(#ff0000);

    specIndex++;
    if (specIndex > spectrogram.height) {
      specIndex = 0;
    }
    spectrogram.line(0, specIndex+1, bins*binwidth+binwidth, specIndex+1);
    spectrogram.endDraw();
  }

  stroke(#ff3300);
  noFill();
  line(0, 0, binwidth*bins, 0);
  
  line(fftband1_start*binwidth, 0, fftband1_start*binwidth,  fftDrawHeight);
  line((fftband1_start+fftband1_width)*binwidth, 0,(fftband1_start+fftband1_width)*binwidth,  fftDrawHeight);
  
  line(fftband2_start*binwidth, 0, fftband2_start*binwidth,  fftDrawHeight);
  line((fftband2_start+fftband2_width)*binwidth, 0,(fftband2_start+fftband2_width)*binwidth,  fftDrawHeight);
  
  stroke(#ff3300);

  
  
  fill(#ffffff);
  circle((fftband1_start+fftband1_width/2)*binwidth+20, 200+20_150+20, map(fftband1_force, 0, 200, 10, 40));

  popMatrix();
}
